#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Extract short video clip.
# Extract to frames:
#   ffmpeg -i holly.ogv -s 256x72 holly%02d.png
# Open Gimp, "Open as Layers", shift-select all.
# Remove duplicate layers.
# Convert to indexed, 1-bit, reduced bleeding dithering.
# Save as holly.gif.
# convert holly.gif holly-bw%02d.png
# ./marquee.py holly-bw*.png > holly.txt

import cgi
import getopt
import sys

import Image

def process_quadrants(im):
    CHARS = [u"\u00a0", u"\u2596", u"\u2597", u"\u2584", u"\u2598", u"\u258c", u"\u259a", u"\u2599",
        u"\u259d", u"\u259e", u"\u2590", u"\u259f", u"\u2580", u"\u259b", u"\u259c", u"\u2588"]
    frame = []
    w, h = im.size
    y = 0
    while y + 1 < h:
        line = []
        x = 0
        while x + 1 < w:
            b0 = im.getpixel((x, y + 1)) and 1 or 0
            b1 = im.getpixel((x + 1, y + 1)) and 1 or 0
            b2 = im.getpixel((x, y)) and 1 or 0
            b3 = im.getpixel((x + 1, y)) and 1 or 0
            i = 8 * b3 + 4 * b2 + 2 * b1 + b0
            i = (~i) & 0xf
            line.append(CHARS[i])
            x += 2
        line = u"".join(line)
        frame.append(line)
        y += 2
    return frame

def process_pillars(im):
    CHARS = [u"\u00a0", u"\u2581", u"\u2582", u"\u2583", u"\u2584", u"\u2585", u"\u2586", u"\u2587", u"\u2588"]
    CHARS.reverse()
    frame = []
    w, h = im.size
    y = 0
    while y < h:
        line = []
        x = 0
        while x < w:
            i = int(im.getpixel((x, y)) / 256.0 * len(CHARS))
            line.append(CHARS[i])
            x += 1
        line = u"".join(line)
        frame.append(line)
        y += 1
    return frame

def html(s):
    return cgi.escape(s)

MODES = {
    "pillars": process_pillars,
    "quadrants": process_quadrants,
}

PROCESS = process_quadrants

opts, args = getopt.gnu_getopt(sys.argv[1:], "m:", ["mode="])
for o, a in opts:
    if o == "-m" or o == "--mode":
        PROCESS = MODES[a]

filenames = args

frames = []
for fn in filenames:
    im = Image.open(fn)
    frame = PROCESS(im)
    frames.append(frame)

print len(frames[0][0])
for y in range(len(frames[0])):
    line = []
    for f in frames:
        line.append(html(f[y]))
    print u"".join(line).encode("utf-8")
